function distance(x0, y0, x1, y1) {
	return (x0 - x1) * (x0 - x1) + (y0 - y1) * (y0 - y1);
}
function crop(img, {left, top, right, bottom}) {
	var dpr = window.devicePixelRatio;
	left *= dpr;
	top *= dpr;
	right *= dpr;
	bottom *= dpr;
	let canvas = document.createElement("canvas");
	canvas.width = right - left;
	canvas.height = bottom - top;
	let ctx = canvas.getContext("2d");
	ctx.drawImage(img, -left, -top);
	return canvas;
}
function newRect() {
	return {
		x0: 0,
		y0: 0,
		x1: window.innerWidth,
		y1: window.innerHeight,
	};
}
var vue = new Vue({
	el: "#app",
	data: function () {
		return {
			nodes: null,
			cur: null,
			f: {
				v: true, // 匹配view
				t: true, // 匹配文字
				p: true, // 绝对路径
				d: null, // 偏移
				text: "",
			},
			text: "录制",
			path: [],
			pkg: "",
			cls: "",
			imageMode: 0,
			image: "",
			step: 0, // [未开始,待选择]
			rect: newRect(),
		};
	},
	computed: {
		list() {
			if (!this.nodes) return null;
			return this.nodes.sort((a, b) => {
				if (!a.area) a.area = (a.right - a.left) * (a.bottom - a.top);
				if (!b.area) b.area = (b.right - b.left) * (b.bottom - b.top);
				return b.area - a.area;
			});
		},
		clickData() {
			if (!this.cur) return null;
			let data = {};
			var ok = false;
			if (this.f.v) {
				ok = true;
				data.view = this.cur.view;
			}
			if (this.f.t) {
				ok = true;
				data.text = this.f.text;
			}
			if (this.f.p) {
				ok = true;
				data.path = this.f.path;
			}
			if (this.f.d && this.f.d.ok && (this.f.d.dx || this.f.d.dy)) {
				let dpr = window.devicePixelRatio;
				data.dx = this.f.d.dx * dpr;
				data.dy = this.f.d.dy * dpr;
			}
			return ok ? data : null;
		},
		clickList() {
			if (!this.clickData) return [];
			let data = this.clickData;
			return this.nodes.filter((x) => {
				if (data.view && x.view != data.view) return false;
				if (data.text && x.text != data.text) return false;
				return true;
			});
		},
		svgPath() {
			if (!this.cur || !this.f.d) return "";
			let dpr = window.devicePixelRatio;
			let s = "M";
			let x = (this.cur.left + this.cur.right) / 2 / dpr;
			let y = (this.cur.top + this.cur.bottom) / 2 / dpr;
			s += x + "," + y;
			let dx = this.f.d.dx;
			let dy = this.f.d.dy;
			s += "l";
			s += dx + "," + dy;
			return s;
		},
		rect1() {
			let {x0, y0, x1, y1} = this.rect;
			let left = Math.max(Math.min(x0, x1), 0);
			let right = Math.min(Math.max(x0, x1), window.innerWidth);
			let top = Math.max(Math.min(y0, y1), 0);
			let bottom = Math.min(Math.max(y0, y1), window.innerHeight);
			return {left, top, right, bottom};
		},
		rectStyle() {
			let {left, top, right, bottom} = this.rect1;
			return {
				left: left + "px",
				top: top + "px",
				width: right - left + "px",
				height: bottom - top + "px",
			};
		},
	},
	watch: {
		cur() {
			if (this.cur) {
				this.f.v = !!this.cur.view;
				this.f.t = !!this.cur.text;
				this.f.p = !(this.f.v || this.f.t);
				this.f.text = this.cur.text;
				this.f.path = this.cur.path;
				this.f.d = null;
			}
		},
	},
	methods: {
		emit(type, msg) {
			console.log(type);
			we.emit(type, (msg || "") + "");
		},
		nodeStyle(node) {
			let dpr = window.devicePixelRatio;
			return {
				left: node.left / dpr + "px",
				top: node.top / dpr + "px",
				width: (node.right - node.left) / dpr + "px",
				height: (node.bottom - node.top) / dpr + "px",
				background: this.cur == node ? "rgba(76,175,80,.7)" : null,
			};
		},
		begin() {
			this.emit("getNodes", "7");
			this.imageMode = 0;
		},
		next(flag) {
			if (this.imageMode) return this.nextImage(flag);
			if (flag < 3 && !this.clickList.length) return;
			let data = {};
			if (this.cls) data.cls = this.cls;
			if (flag < 3) {
				Object.assign(data, this.clickData);
				this.path.push(this.clickData);
			}
			this.text = "继续";
			if (flag > 1) {
				data.end = 1;
				this.text = "录制";
			}
			this.cur = null;
			data = JSON.stringify(data);
			if (data.length < 3) data = "";
			we.emit("next", data);
		},
		nextImage(flag) {
			let data = {};
			if (this.cls) data.cls = this.cls;
			if (flag < 3) {
				data.image = crop(document.querySelector(".bg"), this.rect1).toDataURL();
			}
			this.text = "继续";
			if (flag > 1) {
				data.end = 1;
				this.text = "录制";
			}
			this.cur = null;
			data = JSON.stringify(data);
			if (data.length < 3) data = "";
			we.emit("next", data);
		},
		stop(flag) {
			this.text = "录制";
			if (!flag && this.cur) this.cur = null;
			else {
				this.cur = null;
				this.nodes = null;
				this.step = 0;
				we.emit("stop", "");
			}
		},
		close() {
			we.emit("close", "");
		},
		setDx(e) {
			if (this.f.d && this.cur) {
				let dpr = window.devicePixelRatio;
				this.f.d.dx = Math.floor(e.clientX - (this.cur.left + this.cur.right) / 2 / dpr);
				this.f.d.dy = Math.floor(e.clientY - (this.cur.top + this.cur.bottom) / 2 / dpr);
			}
		},
		getImage(e) {
			this.imageMode = 1;
			this.image = "";
			this.rect = newRect();
			setTimeout(function () {
				we.emit("getImage", "");
			}, 300);
		},
		rectDown(e) {
			e = e.touches[0];
			let x = e.clientX;
			let y = e.clientY;
			let {x0, y0, x1, y1} = this.rect;
			this._point = distance(x, y, x1, y1) < distance(x, y, x0, y0) ? 1 : 0;
			this._px = x;
			this._py = y;
		},
		rectMove(e) {
			e = e.touches[0];
			let x = e.clientX;
			let y = e.clientY;
			this.rect["x" + this._point] += x - this._px;
			this.rect["y" + this._point] += y - this._py;
			this._px = x;
			this._py = y;
		},
	},
	mounted: function () {},
	directives: {
		drag: {
			// 指令的定义
			bind: function (el, d) {
				let divx;
				let divy;
				var k = d.rawName.split(".")[1];
				var storage = localStorage || sessionStorage;
				if (k) {
					k = d.rawName;
					var s = storage.getItem(k);
					if (s) {
						var ss = s.split(",");
						el.style.left = ss[0] + "px";
						el.style.top = ss[1] + "px";
					}
				}
				function touchstart(e) {
					e = e.touches[0];
					//获取鼠标点击处分别与div左边和上边的距离：鼠标位置-div位置
					divx = e.clientX - el.offsetLeft;
					divy = e.clientY - el.offsetTop;
					document.addEventListener("touchmove", touchmove);
					document.addEventListener("touchend", touchend);
				}
				function touchmove(e) {
					e = e.touches[0];
					//获取移动后div的位置：鼠标位置-divx/divy
					let l = e.clientX - divx;
					let t = e.clientY - divy;
					el.style.left = l + "px";
					el.style.top = t + "px";
					if (k) storage.setItem(k, l + "," + t);
				}
				function touchend(e) {
					document.removeEventListener("touchmove", touchmove);
					document.removeEventListener("touchend", touchend);
				}
				el._destroy = function () {
					el.removeEventListener("touchstart", touchstart);
					touchend();
				};
				el.addEventListener("touchstart", touchstart);
			},
			unbind(el) {
				if (el && el._destroy) el._destroy();
			},
		},
	},
});
